import re
from typing import Iterator, List, Optional

from bs4 import BeautifulSoup, NavigableString, Tag


def find_string_after_tag(t: Tag, text: str):
    return (
        isinstance(t.next_element, NavigableString)
        and t.next_element is not None
        and re.search(text, t.next_element)
    )


def sections(t: Tag):
    # Sec 9
    # SEC. 9
    # SEC.. 9
    #  SEC. 9
    #  SEC . 9
    # SEC, 9
    return find_string_after_tag(t, r"^\s*S(EC|ec)[\s\.\,]*\d")


def slice_html(b: BeautifulSoup, items: List[Tag], count: int) -> str:
    body = str(b)
    item = items[count]
    starter_text = str(item)
    start = body.index(starter_text)
    try:
        next_item = items[count + 1]
        ender_text = str(next_item)
        end = body.index(ender_text)
    except:
        end = len(body)
    return body[start:end]


def elements(
    html: BeautifulSoup, tags: List[Tag], container: Optional[int]
) -> Iterator:
    return (
        {
            "unit": slice_html(html, tags, i),
            "container": container,
            "order": int(tags[i]["id"]),
        }
        for i in range(len(tags))
    )


def fetch_units(html: BeautifulSoup, containers: Optional[List[Tag]]):
    items = html(sections)

    for index, parent_tag in enumerate(items, start=2):
        # can start with 2, section 1 previously processed
        parent_tag["id"] = f"{index}"
        parent_tag["data-type"] = "unit"

    unit_list = []
    if not containers:
        units = html("p", attrs={"data-type": "unit"})
        unit_list = list(elements(html, units, None))
    else:
        for counter in range(len(containers)):
            sub_html = BeautifulSoup(
                slice_html(html, containers, counter), "html5lib"
            )
            units = sub_html("p", attrs={"data-type": "unit"})
            unit_list += list(elements(sub_html, units, counter))

    return html, sorted(unit_list, key=lambda k: k["order"])
